<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="utf-8">
  <meta name="viewport" content="initial-scale=1.0">
  <title>RQ dashboard</title>

  {# Le styles -#}
  <link rel="stylesheet" href="{{ url_for('rq_dashboard.static', filename='css/bootstrap.min.css') }}">
  <link href="{{ url_for('rq_dashboard.static', filename='css/main.css') }}" rel="stylesheet">
</head>

<body>

<div class="container">

    {% block content %}{% endblock %}

</div>

{#- JavaScripts are loaded at the end -#}

{#- Custom JS -#}
<script src="{{ url_for('rq_dashboard.static', filename='js/jquery.min.js') }}" type="text/javascript"></script>
<script src="{{ url_for('rq_dashboard.static', filename='js/underscore-min.js') }}" type="text/javascript"></script>
<script src="{{ url_for('rq_dashboard.static', filename='js/sugar-1.2.1.min.js') }}" type="text/javascript"></script>
<script src="{{ url_for('rq_dashboard.static', filename='js/bootstrap-tooltip.js') }}" type="text/javascript"></script>
<script type="text/javascript">
{% block inline_js %}{% endblock %}
</script>

</body>
</html>
